/*
 * --COPYRIGHT--,BSD Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved. Redistribution and use in source and binary
 * forms, with or without modification, are permitted provided that the
 * following conditions are met: * Redistributions of source code must
 * retain the above copyright notice, this list of conditions and the
 * following disclaimer. * Redistributions in binary form must reproduce
 * the above copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided with
 * the distribution. * Neither the name of Texas Instruments Incorporated
 * nor the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission. THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE. --/COPYRIGHT--
 */

//******************************************************************************
//  Jose I Quinones
//  Texas Instruments Inc.
//  May 2011
//  Built with IAR Embedded Workbench Version: 5.1
//******************************************************************************
#include "Config.h"

//int AccelIndex;
//unsigned int AccelArray[2][750];

void Task1(void)
{

  switch (AccelerateState)
  {
  case (NOACC):
    break;
  case (ACCEL):
    if (tmpAccelTimeBase > 0)
        {
        tmpAccelTimeBase -= 1;                  //Wait until delay expires
        }
    else
        {
        if (((unsigned int) StepPeriod - (unsigned int) AccelRate) < DesiredStepperSpeed)
        {
          StepPeriod = DesiredStepperSpeed;
          AccelerateState = NOACC;
          
          //Debug
          //AccelIndex = 0;
          
        }
        else
        {
          StepPeriod -= AccelRate;
          
          //Debug
          //AccelArray[0][AccelIndex] = StepCounter;
          //AccelIndex += 1;
        }
        tmpAccelTimeBase = AccelTimeBase;
        }
    break;

  case (DECEL):
    if (tmpAccelTimeBase > 0)
        {
        tmpAccelTimeBase -= 1;                  //Wait until delay expires
        }
    else
        {
        if (((unsigned int) StepPeriod + (unsigned int) AccelRate) > DesiredStepperSpeed)
        {
          StepPeriod = DesiredStepperSpeed;
          AccelerateState = NOACC;
        }
        else
        {
          StepPeriod += AccelRate;

          //Debug
          //AccelArray[1][AccelIndex] = StepCounter - (AccelSteps + AtSpeedSteps);
          //AccelIndex += 1;
        }
        tmpAccelTimeBase = AccelTimeBase;
        }
    break;

  case (STOP):
    if (tmpAccelTimeBase > 0)
        {
        tmpAccelTimeBase -= 1;                  //Wait until delay expires
        }
    else
        {
        if (((unsigned int) StepPeriod + (unsigned int) AccelRate) < DesiredStepperSpeed)
        {
          StepPeriod += AccelRate;
        }
        else
        {
          TBCCTL6 &= 0xFF0F;                                          //Disable CC and Interrupt
          AccelerateState = NOACC;
          if (WithDisable)
            {
              PHASEA_PORT = 0;
              PHASEB_PORT = 0;
            }
        }
        tmpAccelTimeBase = AccelTimeBase;
        }
    break;

//*************************    S CURVE ACELERATION       ***********************    
    
//******************************************************************************
// S Curve Acceleration Increase
// Changes the Acceleration Rate by increasing it until the SCurveAccelSpeed is met.
//******************************************************************************    
      case (S_CURVE_ACCELI):
    if (tmpAccelTimeBase > 0)
        {
        tmpAccelTimeBase -= 1;                  //Wait until delay expires
        }
    else
        {
        if ((unsigned int) StepPeriod < (unsigned int)SCurveAccelSpeed)
          {
          AccelerateState = S_CURVE_ACCSS;
          }
        else
          {
          AccelRate += AccelRateChange;
          }
        StepPeriod -= AccelRate;
        tmpAccelTimeBase = AccelTimeBase;
        }
    break;
//******************************************************************************
// S Curve Acceleration Steady State
// Acceleration Rate is unmodified until the SCurveDecelSpeed is met.
//******************************************************************************    
      case (S_CURVE_ACCSS):
    if (tmpAccelTimeBase > 0)
        {
        tmpAccelTimeBase -= 1;                  //Wait until delay expires
        }
    else
        {
        if ((unsigned int) StepPeriod < (unsigned int)SCurveDecelSpeed)
          {
          AccelerateState = S_CURVE_ACCELD;
          AccelRate -= AccelRateChange;
          }
        StepPeriod -= AccelRate;
        tmpAccelTimeBase = AccelTimeBase;
        }
    break;
//******************************************************************************
// S Curve Acceleration Decrease
// Changes the Acceleration Rate by decreasing it until the DesiredSpeed is met.
//******************************************************************************      
      case (S_CURVE_ACCELD):
    if (tmpAccelTimeBase > 0)
        {
        tmpAccelTimeBase -= 1;                  //Wait until delay expires
        }
    else
        {
        if ((unsigned int) StepPeriod < (unsigned int)DesiredStepperSpeed)
          {
          StepPeriod = DesiredStepperSpeed;
          AccelerateState = NOACC;
          }
        else
          {
          StepPeriod -= AccelRate;
          AccelRate -=AccelRateChange;
          }
        tmpAccelTimeBase = AccelTimeBase;
        }
    break;

//*************************    S CURVE DECCELERATION     ***********************    
    
//******************************************************************************
// S Curve Deceleration Increase
// Changes the Acceleration Rate by increasing it until the SCurveDecelSpeed is met.
//******************************************************************************    
      case (S_CURVE_DECELI):
    if (tmpAccelTimeBase > 0)
        {
        tmpAccelTimeBase -= 1;                  //Wait until delay expires
        }
    else
        {
        if ((unsigned int) StepPeriod > (unsigned int)SCurveDecelSpeed)
          {
          AccelerateState = S_CURVE_DECSS;
          }
        else
          {
          AccelRate += AccelRateChange;
          }
        StepPeriod += AccelRate;
        tmpAccelTimeBase = AccelTimeBase;
        }
    break;
//******************************************************************************
// S Curve Acceleration Steady State
// Acceleration Rate is unmodified until the SCurveAccelSpeed is met.
//******************************************************************************    
      case (S_CURVE_DECSS):
    if (tmpAccelTimeBase > 0)
        {
        tmpAccelTimeBase -= 1;                  //Wait until delay expires
        }
    else
        {
        if ((unsigned int) StepPeriod > (unsigned int)SCurveAccelSpeed)
          {
          AccelerateState = S_CURVE_DECELD;
          AccelRate -= AccelRateChange;
          }
        StepPeriod += AccelRate;
        tmpAccelTimeBase = AccelTimeBase;
        }
    break;
//******************************************************************************
// S Curve Acceleration Decrease
// Changes the Acceleration Rate by decreasing it until the DesiredSpeed is met.
//******************************************************************************      
      case (S_CURVE_DECELD):
    if (tmpAccelTimeBase > 0)
        {
        tmpAccelTimeBase -= 1;                  //Wait until delay expires
        }
    else
        {
        if ((unsigned int) StepPeriod > (unsigned int)DesiredStepperSpeed)
        {
          StepPeriod = DesiredStepperSpeed;
          AccelerateState = NOACC;
          TBCCTL6 &= 0xFF0F;                                          //Disable CC and Interrupt
        }
        else
        {
          StepPeriod += AccelRate;
          AccelRate -=AccelRateChange;
        }
        tmpAccelTimeBase = AccelTimeBase;
        }
    break;    
    
    
    
  }
}
